/**************************************************************

 myfft provides functions for handling the FFT. Main function is: 

 void fourier_spectrum(Float *sf, Float *fft_data, int fft_size, 
		     int data_len, Float *window, int win_len)

 abs(FFT(sf)) is computed with this function. Requires that FFTW 
 is installed.

***************************************************************/

#include "myfft.h"
#include "params.h"
#include <iostream>
#include <fstream>
using namespace std;

static double *rdata = NULL, *idata = NULL;
static fftw_plan rplan, iplan;
static int last_fft_size = 0;
static Float w_rate = (TWO_PI / 22050.0);


/*--------------------------------------------

 Conversion functions

---------------------------------------------*/

Float mus_linear_to_db(Float x) {
	if (x > 0.0) return(20.0 * log10(x)); 
	return(-200.0);
}

Float mus_radians_to_hz(Float rads) {return(rads / w_rate);}
Float mus_hz_to_radians(Float hz) {return(hz * w_rate);}
Float mus_degrees_to_radians(Float degree) {return(degree * TWO_PI / 360.0);}
Float mus_radians_to_degrees(Float rads) {return(rads * 360.0 / TWO_PI);}
Float mus_db_to_linear(Float x) {return(pow(10.0, x / 20.0));}


/*--------------------------------------------

 IsNaN returns 1 if the input number is a NaN.

-------------------------------------------*/

bool isNaN(double val) {
	return (isnan(val) || isinf(val));
}

/*---------------------------------------------

 Create Hanning window

----------------------------------------------*/

void Hanning(Float32 *window, int size) {
	for (int i=1; i<=size; i++)
		window[i-1]=0.5*(1-cos(2*M_PI*((double)i/(double)(size+1))));
}

void Hamhan(Float *window, int size) {
	for (int i=1; i<=size; i++)
		window[i-1]=pow(0.5-0.5*(cos(2*M_PI*((double)i/(double)(size)))),2.0);
}

/* Create BlackmanHarris92 dB */

void Blackman92(Float *window, int size)
{
// function to create a backmanHarris window

	int     i;
	double fSum = 0;

	// for -92dB 
	float a0 = .35875, a1 = .48829, a2 = .14128, a3 = .01168;
	double fConst = TWO_PI / (size-1);
  
        // compute window
        for(i = 0; i < size; i++)
        {
                fSum += window[i] = a0 - a1 * cos(fConst * i) +
                        a2 * cos(fConst * 2 * i) - a3 * cos(fConst * 3 * i);
        }

        // I do not know why I now need this factor of two
        fSum = fSum / 2.0;  // AVISAR DE ESTO!!

        // scale function
        for (i = 0; i < size; i++)
                window[i] = 500.0*window[i] / fSum;
}


/*--------------------------------------------

 Apply FFTW

---------------------------------------------*/

void mus_fftw(Float32 *rl, int n, int dir)
{
  int i;
  if (n != last_fft_size)
    {
      if (rdata) {fftw_free(rdata); fftw_free(idata); fftw_destroy_plan(rplan); fftw_destroy_plan(iplan);}
      rdata = (double *)fftw_malloc(n * sizeof(double));
      idata = (double *)fftw_malloc(n * sizeof(double));
      rplan = fftw_plan_r2r_1d(n, rdata, idata, FFTW_R2HC, FFTW_ESTIMATE); 
      iplan = fftw_plan_r2r_1d(n, rdata, idata, FFTW_HC2R, FFTW_ESTIMATE);
      last_fft_size = n;
    }
  memset((void *)idata, 0, n * sizeof(double));
  for (i = 0; i < n; i++) {rdata[i] = rl[i];}
  if (dir != -1)
    fftw_execute(rplan);
  else fftw_execute(iplan);
  for (i = 0; i < n; i++) rl[i] = idata[i];
}

/*---------------------------------------------
 
 Spectrum calculation, using mus_fftw

---------------------------------------------*/

void fourier_spectrum(Float32 sf[], Float32 fft_data[], Float32 fft_phase[], int fft_size, int data_len, Float32 window[], int win_len)
{
	int i;
	for (i = 0; i < win_len; i++)
	  
	fft_data[i] = window[i] * sf[i]; 
		
	int j;
	mus_fftw(fft_data, fft_size, 1);

	fft_data[0] = fabs(fft_data[0]);
	fft_data[fft_size / 2] = fabs(fft_data[fft_size / 2]);

	for (i = 1, j = fft_size - 1; i < fft_size / 2; i++, j--) 
	{
		fft_data[i] = hypot(fft_data[i], fft_data[j]);
		fft_phase[i]=atan(fft_data[j]/fft_data[i]);
	}

	for (int i=0; i<fft_size/2; i++)
	if (isNaN(fft_data[i]))
		fft_data[i]=0.0;
 }

